/* -*- coding: utf-8 -*- */
/** \file OPiQuotations.js
 * (October 25, 2015)
 *
 * \brief
 * JavaScript functions to
 *   - highlight founded words
 *   - deal AJAX lists
 *   - init sharing buttons.
 *
 * Piece of OPiQuotations.
 * https://bitbucket.org/OPiMedia/opiquotations
 *
 * GPLv3 --- Copyright (C) 2014, 2015 Olivier Pirson
 * http://www.opimedia.be/
 */

/**
 * \brief
 * true if highlight is active,
 * else false.
 */
var highlight_active = false;


/**
 * \brief
 * The stylesheet to modify.
 */
var highlight_sheet = null;


/**
 * \brief
 * Add a CSS class to item.
 *
 * @param DOM item
 * @param String class_name
 */
function add_css_class(item, class_name) {
    "use strict";

    if (class_name !== '') {
        try {
            item.classList.add(class_name);
        }
        catch (e) {  // some old browsers
            item.className += ' ' + class_name;
        }
    }
}


/**
 * \brief
 * AJAX load the list name, if necessary.
 *
 * @param String name
 */
function build_list(name) {
    "use strict";

    if (document.getElementById('list-' + name + '-list') !== null) {  // already loaded
        return;
    }

    var xhr = (window.XMLHttpRequest
               ? new XMLHttpRequest()
               : new ActiveXObject('Microsoft.XMLHTTP'));  // old IE

    xhr.open('GET', 'OPiQuotations_list.php?list=' + name + 's', true);
    xhr.setRequestHeader('Content-Type', 'application/json;charset=UTF-8');

    xhr.onreadystatechange = function() {  // callback
        if ((xhr.readyState !== 4) || (xhr.status !== 200)) {
            return;
        }

        var list;

        try {
            list = JSON.parse(xhr.responseText);
        }
        catch (e) {  // some old browsers
            list = eval('(' + xhr.responseText + ')');
        }

        // Calculate min and max of occurrences
        var nb;
        var min = Number.MAX_VALUE;
        var max = 0;
        var i;

        for (i in list) {
            nb = list[i][1];
            min = Math.min(min, nb);
            max = Math.max(max, nb);
        }

        // Build the list
        var ul_item = document.createElement('ul');

        ul_item.setAttribute('id', 'list-' + name + '-list');

        var a_item;
        var value;
        var percent;

        for (i in list) {
            a_item = document.createElement('a');

            value = list[i][0];
            nb = list[i][1];

            percent = (nb - min)*120/(max - min) + 80;  // between 80% and 200%

            add_css_class(a_item, (nb === min
                                   ? 'min'
                                   : (nb === max
                                      ? 'max'
                                      : '')));

            a_item.setAttribute('rel', 'nofollow');
            a_item.setAttribute('href', encodeURI('?' + name + '=' + value));
            a_item.setAttribute('title', nb + ' citation' + (nb > 1
                                                             ? 's'
                                                             : ''));
            a_item.style.fontSize = Math.round(percent) + '%';

            a_item.appendChild(document.createTextNode(value));

            ul_item.appendChild(document.createElement('li')).appendChild(a_item);
        }

        var container_item = document.getElementById('list-' + name + '-container');

        container_item.innerHTML = '';  // remove loading message
        container_item.appendChild(ul_item);
    };

    xhr.send(null);
}


/**
 * \brief
 * Switch on/off all highlighted words.
 */
function highlight_on_off() {
    "use strict";

    if (highlight_sheet !== null) {
        if (highlight_active) {
            highlight_sheet.deleteRule(0);
            highlight_sheet.deleteRule(0);
        }
        else {
            highlight_sheet.insertRule('span.highlight { background-color: yellow; margin: -2px -1px; padding: 2px 1px; }', 0);
            highlight_sheet.insertRule('section.maxim span.highlight { background-color: #ffbda0; }', 1);
        }
        highlight_active = !highlight_active;
    }
}


/**
 * \brief
 * Version of highlight_on_off() to Internet Explorer < 9.
 */
function highlight_on_off_ie_before_9() {
    "use strict";

    if (highlight_sheet !== null) {
        if (highlight_active) {
            highlight_sheet.removeRule(0);
        }
        else {
            highlight_sheet.addRule('span.highlight', 'background-color: yellow', 0);
        }
        highlight_active = !highlight_active;
    }
}


/**
 * \brief
 * Close all lists in panel,
 * open list of name.
 *
 * Load the list name, if necessary.
 *
 * @param String name
 */
function list_open(name) {
    "use strict";

    lists_hide();

    window.scroll(0, 0);
    document.getElementById('list-' + name).style.display = 'block';

    build_list(name);
}


/**
 * \brief
 * Close all lists in panel.
 */
function lists_hide() {
    "use strict";

    var labels = ['author', 'nation', 'subject', 'work'];

    var i;

    for (i in labels) {
        document.getElementById('list-' + labels[i]).style.display = 'none';
    }
}


/**
 * \brief
 * Polyfill addEventListener().
 *
 * @param String event
 * @param function func
 */
function opi_addEventListener(event, func) {
    "use strict";

    if (window.addEventListener) {
        window.addEventListener(event, func);
    } else if (window.attachEvent) {  // old IE
        window.attachEvent('on' + event, func);
    }
}



/**
 * \brief
 * Initializations.
 */
(function () {
    "use strict";

    // Search the good stylesheet
    var i;
    for (i = 0; i < document.styleSheets.length; i++) {
        highlight_sheet = document.styleSheets[i];

        if (highlight_sheet.title === 'highlight') {
            break;
        }
    }

    // Replace the highlight_on_off() function if Internet Explorer < 9
    if (!highlight_sheet.insertRule) {
        highlight_on_off = highlight_on_off_ie_before_9;
    }


    // Facebook sharing button
    window.fbAsyncInit = function () {
        FB.init({appId:   '164818953866160',
                 xfbml:   true,
                 version: 'v2.5'
                });
    };


    // Onload
    opi_addEventListener('load', function () {
        // Google+ sharing button
        if (typeof gapi !== 'undefined') {
            gapi.plus.go(document.getElementById('sharing-global'));
        }
    });
}());
